@@ -31,18 +31,10 @@ def health_check(connection, request):
 
 
 async def main():
-    # Set the stop condition when receiving SIGTERM.
-    loop = asyncio.get_running_loop()
-    stop = loop.create_future()
-    loop.add_signal_handler(signal.SIGTERM, stop.set_result, None)
-
-    async with serve(
-        slow_echo,
-        host="",
-        port=80,
-        process_request=health_check,
-    ):
-        await stop
+    async with serve(slow_echo, "", 80, process_request=health_check) as server:
+        loop = asyncio.get_running_loop()
+        loop.add_signal_handler(signal.SIGTERM, server.close)
+        await server.wait_closed()
 
 
 if __name__ == "__main__":
