@@ -17,6 +17,7 @@ from ..datastructures import Headers, HeadersLike
 from ..exceptions import (
     InvalidHeader,
     InvalidHeaderValue,
+    InvalidMessage,
     NegotiationError,
     SecurityError,
 )
@@ -34,7 +35,7 @@ from ..headers import (
 from ..http11 import USER_AGENT
 from ..typing import ExtensionHeader, LoggerLike, Origin, Subprotocol
 from ..uri import WebSocketURI, parse_uri
-from .exceptions import InvalidMessage, InvalidStatusCode, RedirectHandshake
+from .exceptions import InvalidStatusCode, RedirectHandshake
 from .handshake import build_request, check_response
 from .http import read_response
 from .protocol import WebSocketCommonProtocol
