@@ -2,9 +2,12 @@ import asyncio
 import contextlib
 import http
 import logging
+import os
 import socket
 import ssl
+import sys
 import unittest
+from unittest.mock import patch
 
 from websockets.asyncio.client import *
 from websockets.asyncio.compatibility import TimeoutError
@@ -12,12 +15,17 @@ from websockets.asyncio.server import serve, unix_serve
 from websockets.client import backoff
 from websockets.exceptions import (
     InvalidHandshake,
+    InvalidMessage,
+    InvalidProxy,
+    InvalidProxyMessage,
     InvalidStatus,
     InvalidURI,
+    ProxyError,
     SecurityError,
 )
 from websockets.extensions.permessage_deflate import PerMessageDeflate
 
+from ..proxy import ProxyMixin
 from ..utils import CLIENT_CONTEXT, MS, SERVER_CONTEXT, temp_unix_socket_path
 from .server import args, get_host_port, get_uri, handler
 
@@ -75,6 +83,21 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
                 async with connect("ws://invalid/", sock=sock) as client:
                     self.assertEqual(client.protocol.state.name, "OPEN")
 
+    async def test_compression_is_enabled(self):
+        """Client enables compression by default."""
+        async with serve(*args) as server:
+            async with connect(get_uri(server)) as client:
+                self.assertEqual(
+                    [type(ext) for ext in client.protocol.extensions],
+                    [PerMessageDeflate],
+                )
+
+    async def test_disable_compression(self):
+        """Client disables compression."""
+        async with serve(*args) as server:
+            async with connect(get_uri(server), compression=None) as client:
+                self.assertEqual(client.protocol.extensions, [])
+
     async def test_additional_headers(self):
         """Client can set additional headers with additional_headers."""
         async with serve(*args) as server:
@@ -95,20 +118,13 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
             async with connect(get_uri(server), user_agent_header=None) as client:
                 self.assertNotIn("User-Agent", client.request.headers)
 
-    async def test_compression_is_enabled(self):
-        """Client enables compression by default."""
+    async def test_legacy_user_agent(self):
+        """Client can override User-Agent header with additional_headers."""
         async with serve(*args) as server:
-            async with connect(get_uri(server)) as client:
-                self.assertEqual(
-                    [type(ext) for ext in client.protocol.extensions],
-                    [PerMessageDeflate],
-                )
-
-    async def test_disable_compression(self):
-        """Client disables compression."""
-        async with serve(*args) as server:
-            async with connect(get_uri(server), compression=None) as client:
-                self.assertEqual(client.protocol.extensions, [])
+            async with connect(
+                get_uri(server), additional_headers={"User-Agent": "Smith"}
+            ) as client:
+                self.assertEqual(client.request.headers["User-Agent"], "Smith")
 
     async def test_keepalive_is_enabled(self):
         """Client enables keepalive and measures latency by default."""
@@ -151,22 +167,24 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
         iterations = 0
         successful = 0
 
-        def process_request(connection, request):
+        async def process_request(connection, request):
             nonlocal iterations
             iterations += 1
             # Retriable errors
             if iterations == 1:
-                connection.transport.close()
+                await asyncio.sleep(3 * MS)
             elif iterations == 2:
+                connection.transport.close()
+            elif iterations == 3:
                 return connection.respond(http.HTTPStatus.SERVICE_UNAVAILABLE, "🚒")
             # Fatal error
-            elif iterations == 5:
+            elif iterations == 6:
                 return connection.respond(http.HTTPStatus.PAYMENT_REQUIRED, "💸")
 
         async with serve(*args, process_request=process_request) as server:
             with self.assertRaises(InvalidStatus) as raised:
                 async with short_backoff_delay():
-                    async for client in connect(get_uri(server)):
+                    async for client in connect(get_uri(server), open_timeout=3 * MS):
                         self.assertEqual(client.protocol.state.name, "OPEN")
                         successful += 1
 
@@ -174,7 +192,7 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
             str(raised.exception),
             "server rejected WebSocket connection: HTTP 402",
         )
-        self.assertEqual(iterations, 5)
+        self.assertEqual(iterations, 6)
         self.assertEqual(successful, 2)
 
     async def test_reconnect_with_custom_process_exception(self):
@@ -245,7 +263,7 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
 
         async with serve(*args, process_request=redirect) as server:
             async with connect(get_uri(server) + "/redirect") as client:
-                self.assertEqual(client.protocol.wsuri.path, "/")
+                self.assertEqual(client.protocol.uri.path, "/")
 
     async def test_cross_origin_redirect(self):
         """Client follows redirect to a secure URI on a different origin."""
@@ -294,7 +312,7 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
             async with connect(
                 "ws://overridden/redirect", host=host, port=port
             ) as client:
-                self.assertEqual(client.protocol.wsuri.path, "/")
+                self.assertEqual(client.protocol.uri.path, "/")
 
     async def test_cross_origin_redirect_with_explicit_host_port(self):
         """Client doesn't follow cross-origin redirect with an explicit host / port."""
@@ -367,37 +385,34 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
 
     async def test_timeout_during_handshake(self):
         """Client times out before receiving handshake response from server."""
-        gate = asyncio.get_running_loop().create_future()
-
-        async def stall_connection(self, request):
-            await gate
-
-        # The connection will be open for the server but failed for the client.
-        # Use a connection handler that exits immediately to avoid an exception.
-        async with serve(*args, process_request=stall_connection) as server:
-            try:
-                with self.assertRaises(TimeoutError) as raised:
-                    async with connect(get_uri(server) + "/no-op", open_timeout=2 * MS):
-                        self.fail("did not raise")
-                self.assertEqual(
-                    str(raised.exception),
-                    "timed out during handshake",
-                )
-            finally:
-                gate.set_result(None)
+        # Replace the WebSocket server with a TCP server that doesn't respond.
+        with socket.create_server(("localhost", 0)) as sock:
+            host, port = sock.getsockname()
+            with self.assertRaises(TimeoutError) as raised:
+                async with connect(f"ws://{host}:{port}", open_timeout=MS):
+                    self.fail("did not raise")
+            self.assertEqual(
+                str(raised.exception),
+                "timed out during opening handshake",
+            )
 
     async def test_connection_closed_during_handshake(self):
         """Client reads EOF before receiving handshake response from server."""
 
         def close_connection(self, request):
-            self.close_transport()
+            self.transport.close()
 
         async with serve(*args, process_request=close_connection) as server:
-            with self.assertRaises(EOFError) as raised:
+            with self.assertRaises(InvalidMessage) as raised:
                 async with connect(get_uri(server)):
                     self.fail("did not raise")
             self.assertEqual(
                 str(raised.exception),
+                "did not receive a valid HTTP response",
+            )
+            self.assertIsInstance(raised.exception.__cause__, EOFError)
+            self.assertEqual(
+                str(raised.exception.__cause__),
                 "connection closed while reading HTTP status line",
             )
 
@@ -443,11 +458,16 @@ class ClientTests(unittest.IsolatedAsyncioTestCase):
         server = await asyncio.start_server(junk, "localhost", 0)
         host, port = get_host_port(server)
         async with server:
-            with self.assertRaises(ValueError) as raised:
+            with self.assertRaises(InvalidMessage) as raised:
                 async with connect(f"ws://{host}:{port}"):
                     self.fail("did not raise")
             self.assertEqual(
                 str(raised.exception),
+                "did not receive a valid HTTP response",
+            )
+            self.assertIsInstance(raised.exception.__cause__, ValueError)
+            self.assertEqual(
+                str(raised.exception.__cause__),
                 "unsupported protocol; expected HTTP/1.1: "
                 "220 smtp.invalid ESMTP Postfix",
             )
@@ -485,7 +505,7 @@ class SecureClientTests(unittest.IsolatedAsyncioTestCase):
         """Client rejects certificate where server certificate isn't trusted."""
         async with serve(*args, ssl=SERVER_CONTEXT) as server:
             with self.assertRaises(ssl.SSLCertVerificationError) as raised:
-                # The test certificate isn't trusted system-wide.
+                # The test certificate is self-signed.
                 async with connect(get_uri(server)):
                     self.fail("did not raise")
             self.assertIn(
@@ -542,6 +562,297 @@ class SecureClientTests(unittest.IsolatedAsyncioTestCase):
         )
 
 
+@unittest.skipUnless("mitmproxy" in sys.modules, "mitmproxy not installed")
+class SocksProxyClientTests(ProxyMixin, unittest.IsolatedAsyncioTestCase):
+    proxy_mode = "socks5@51080"
+
+    @patch.dict(os.environ, {"socks_proxy": "http://localhost:51080"})
+    async def test_socks_proxy(self):
+        """Client connects to server through a SOCKS5 proxy."""
+        async with serve(*args) as server:
+            async with connect(get_uri(server)) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"socks_proxy": "http://localhost:51080"})
+    async def test_secure_socks_proxy(self):
+        """Client connects to server securely through a SOCKS5 proxy."""
+        async with serve(*args, ssl=SERVER_CONTEXT) as server:
+            async with connect(get_uri(server), ssl=CLIENT_CONTEXT) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"socks_proxy": "http://hello:iloveyou@localhost:51080"})
+    async def test_authenticated_socks_proxy(self):
+        """Client connects to server through an authenticated SOCKS5 proxy."""
+        try:
+            self.proxy_options.update(proxyauth="hello:iloveyou")
+            async with serve(*args) as server:
+                async with connect(get_uri(server)) as client:
+                    self.assertEqual(client.protocol.state.name, "OPEN")
+        finally:
+            self.proxy_options.update(proxyauth=None)
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"socks_proxy": "http://localhost:51080"})
+    async def test_authenticated_socks_proxy_error(self):
+        """Client fails to authenticate to the SOCKS5 proxy."""
+        from python_socks import ProxyError as SocksProxyError
+
+        try:
+            self.proxy_options.update(proxyauth="any")
+            with self.assertRaises(ProxyError) as raised:
+                async with connect("ws://example.com/"):
+                    self.fail("did not raise")
+        finally:
+            self.proxy_options.update(proxyauth=None)
+        self.assertEqual(
+            str(raised.exception),
+            "failed to connect to SOCKS proxy",
+        )
+        self.assertIsInstance(raised.exception.__cause__, SocksProxyError)
+        self.assertNumFlows(0)
+
+    @patch.dict(os.environ, {"socks_proxy": "http://localhost:61080"})  # bad port
+    async def test_socks_proxy_connection_failure(self):
+        """Client fails to connect to the SOCKS5 proxy."""
+        from python_socks import ProxyConnectionError as SocksProxyConnectionError
+
+        with self.assertRaises(OSError) as raised:
+            async with connect("ws://example.com/"):
+                self.fail("did not raise")
+        # Don't test str(raised.exception) because we don't control it.
+        self.assertIsInstance(raised.exception, SocksProxyConnectionError)
+        self.assertNumFlows(0)
+
+    async def test_socks_proxy_connection_timeout(self):
+        """Client times out while connecting to the SOCKS5 proxy."""
+        # Replace the proxy with a TCP server that doesn't respond.
+        with socket.create_server(("localhost", 0)) as sock:
+            host, port = sock.getsockname()
+            with patch.dict(os.environ, {"socks_proxy": f"http://{host}:{port}"}):
+                with self.assertRaises(TimeoutError) as raised:
+                    async with connect("ws://example.com/", open_timeout=MS):
+                        self.fail("did not raise")
+        self.assertEqual(
+            str(raised.exception),
+            "timed out during opening handshake",
+        )
+        self.assertNumFlows(0)
+
+    async def test_explicit_socks_proxy(self):
+        """Client connects to server through a SOCKS5 proxy set explicitly."""
+        async with serve(*args) as server:
+            async with connect(
+                get_uri(server),
+                # Take this opportunity to test socks5 instead of socks5h.
+                proxy="socks5://localhost:51080",
+            ) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"socks_proxy": "http://localhost:51080"})
+    async def test_ignore_proxy_with_existing_socket(self):
+        """Client connects using a pre-existing socket."""
+        async with serve(*args) as server:
+            with socket.create_connection(get_host_port(server)) as sock:
+                # Use a non-existing domain to ensure we connect to sock.
+                async with connect("ws://invalid/", sock=sock) as client:
+                    self.assertEqual(client.protocol.state.name, "OPEN")
+        self.assertNumFlows(0)
+
+
+@unittest.skipUnless("mitmproxy" in sys.modules, "mitmproxy not installed")
+class HTTPProxyClientTests(ProxyMixin, unittest.IsolatedAsyncioTestCase):
+    proxy_mode = "regular@58080"
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:58080"})
+    async def test_http_proxy(self):
+        """Client connects to server through an HTTP proxy."""
+        async with serve(*args) as server:
+            async with connect(get_uri(server)) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:58080"})
+    async def test_secure_http_proxy(self):
+        """Client connects to server securely through an HTTP proxy."""
+        async with serve(*args, ssl=SERVER_CONTEXT) as server:
+            async with connect(get_uri(server), ssl=CLIENT_CONTEXT) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+                ssl_object = client.transport.get_extra_info("ssl_object")
+                self.assertEqual(ssl_object.version()[:3], "TLS")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"https_proxy": "http://hello:iloveyou@localhost:58080"})
+    async def test_authenticated_http_proxy(self):
+        """Client connects to server through an authenticated HTTP proxy."""
+        try:
+            self.proxy_options.update(proxyauth="hello:iloveyou")
+            async with serve(*args) as server:
+                async with connect(get_uri(server)) as client:
+                    self.assertEqual(client.protocol.state.name, "OPEN")
+        finally:
+            self.proxy_options.update(proxyauth=None)
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:58080"})
+    async def test_authenticated_http_proxy_error(self):
+        """Client fails to authenticate to the HTTP proxy."""
+        try:
+            self.proxy_options.update(proxyauth="any")
+            with self.assertRaises(ProxyError) as raised:
+                async with connect("ws://example.com/"):
+                    self.fail("did not raise")
+        finally:
+            self.proxy_options.update(proxyauth=None)
+        self.assertEqual(
+            str(raised.exception),
+            "proxy rejected connection: HTTP 407",
+        )
+        self.assertNumFlows(0)
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:58080"})
+    async def test_http_proxy_override_user_agent(self):
+        """Client can override User-Agent header with user_agent_header."""
+        async with serve(*args) as server:
+            async with connect(get_uri(server), user_agent_header="Smith") as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+        [http_connect] = self.get_http_connects()
+        self.assertEqual(http_connect.request.headers[b"User-Agent"], "Smith")
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:58080"})
+    async def test_http_proxy_remove_user_agent(self):
+        """Client can remove User-Agent header with user_agent_header."""
+        async with serve(*args) as server:
+            async with connect(get_uri(server), user_agent_header=None) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+        [http_connect] = self.get_http_connects()
+        self.assertNotIn(b"User-Agent", http_connect.request.headers)
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:58080"})
+    async def test_http_proxy_protocol_error(self):
+        """Client receives invalid data when connecting to the HTTP proxy."""
+        try:
+            self.proxy_options.update(break_http_connect=True)
+            with self.assertRaises(InvalidProxyMessage) as raised:
+                async with connect("ws://example.com/"):
+                    self.fail("did not raise")
+        finally:
+            self.proxy_options.update(break_http_connect=False)
+        self.assertEqual(
+            str(raised.exception),
+            "did not receive a valid HTTP response from proxy",
+        )
+        self.assertNumFlows(0)
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:58080"})
+    async def test_http_proxy_connection_error(self):
+        """Client receives no response when connecting to the HTTP proxy."""
+        try:
+            self.proxy_options.update(close_http_connect=True)
+            with self.assertRaises(InvalidProxyMessage) as raised:
+                async with connect("ws://example.com/"):
+                    self.fail("did not raise")
+        finally:
+            self.proxy_options.update(close_http_connect=False)
+        self.assertEqual(
+            str(raised.exception),
+            "did not receive a valid HTTP response from proxy",
+        )
+        self.assertNumFlows(0)
+
+    @patch.dict(os.environ, {"https_proxy": "http://localhost:48080"})  # bad port
+    async def test_http_proxy_connection_failure(self):
+        """Client fails to connect to the HTTP proxy."""
+        with self.assertRaises(OSError):
+            async with connect("ws://example.com/"):
+                self.fail("did not raise")
+        # Don't test str(raised.exception) because we don't control it.
+        self.assertNumFlows(0)
+
+    async def test_http_proxy_connection_timeout(self):
+        """Client times out while connecting to the HTTP proxy."""
+        # Replace the proxy with a TCP server that doesn't respond.
+        with socket.create_server(("localhost", 0)) as sock:
+            host, port = sock.getsockname()
+            with patch.dict(os.environ, {"https_proxy": f"http://{host}:{port}"}):
+                with self.assertRaises(TimeoutError) as raised:
+                    async with connect("ws://example.com/", open_timeout=MS):
+                        self.fail("did not raise")
+        self.assertEqual(
+            str(raised.exception),
+            "timed out during opening handshake",
+        )
+
+    @patch.dict(os.environ, {"https_proxy": "https://localhost:58080"})
+    async def test_https_proxy(self):
+        """Client connects to server through an HTTPS proxy."""
+        async with serve(*args) as server:
+            async with connect(
+                get_uri(server),
+                proxy_ssl=self.proxy_context,
+            ) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"https_proxy": "https://localhost:58080"})
+    async def test_secure_https_proxy(self):
+        """Client connects to server securely through an HTTPS proxy."""
+        async with serve(*args, ssl=SERVER_CONTEXT) as server:
+            async with connect(
+                get_uri(server),
+                ssl=CLIENT_CONTEXT,
+                proxy_ssl=self.proxy_context,
+            ) as client:
+                self.assertEqual(client.protocol.state.name, "OPEN")
+                ssl_object = client.transport.get_extra_info("ssl_object")
+                self.assertEqual(ssl_object.version()[:3], "TLS")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"https_proxy": "https://localhost:58080"})
+    async def test_https_server_hostname(self):
+        """Client sets server_hostname to the value of proxy_server_hostname."""
+        async with serve(*args) as server:
+            # Pass an argument not prefixed with proxy_ for coverage.
+            kwargs = {"all_errors": True} if sys.version_info >= (3, 12) else {}
+            async with connect(
+                get_uri(server),
+                proxy_ssl=self.proxy_context,
+                proxy_server_hostname="overridden",
+                **kwargs,
+            ) as client:
+                ssl_object = client.transport.get_extra_info("ssl_object")
+                self.assertEqual(ssl_object.server_hostname, "overridden")
+        self.assertNumFlows(1)
+
+    @patch.dict(os.environ, {"https_proxy": "https://localhost:58080"})
+    async def test_https_proxy_invalid_proxy_certificate(self):
+        """Client rejects certificate when proxy certificate isn't trusted."""
+        with self.assertRaises(ssl.SSLCertVerificationError) as raised:
+            # The proxy certificate isn't trusted.
+            async with connect("wss://example.com/"):
+                self.fail("did not raise")
+        self.assertIn(
+            "certificate verify failed: unable to get local issuer certificate",
+            str(raised.exception),
+        )
+
+    @patch.dict(os.environ, {"https_proxy": "https://localhost:58080"})
+    async def test_https_proxy_invalid_server_certificate(self):
+        """Client rejects certificate when proxy certificate isn't trusted."""
+        async with serve(*args, ssl=SERVER_CONTEXT) as server:
+            with self.assertRaises(ssl.SSLCertVerificationError) as raised:
+                # The test certificate is self-signed.
+                async with connect(get_uri(server), proxy_ssl=self.proxy_context):
+                    self.fail("did not raise")
+        self.assertIn(
+            "certificate verify failed: self signed certificate",
+            str(raised.exception).replace("-", " "),
+        )
+        self.assertNumFlows(1)
+
+
 @unittest.skipUnless(hasattr(socket, "AF_UNIX"), "this test requires Unix sockets")
 class UnixClientTests(unittest.IsolatedAsyncioTestCase):
     async def test_connection(self):
@@ -578,10 +889,7 @@ class UnixClientTests(unittest.IsolatedAsyncioTestCase):
             "cannot follow cross-origin redirect to ws://other/ with a Unix socket",
         )
 
-
-@unittest.skipUnless(hasattr(socket, "AF_UNIX"), "this test requires Unix sockets")
-class SecureUnixClientTests(unittest.IsolatedAsyncioTestCase):
-    async def test_connection(self):
+    async def test_secure_connection(self):
         """Client connects to server securely over a Unix socket."""
         with temp_unix_socket_path() as path:
             async with unix_serve(handler, path, ssl=SERVER_CONTEXT):
@@ -615,7 +923,7 @@ class ClientUsageErrorsTests(unittest.IsolatedAsyncioTestCase):
         )
 
     async def test_secure_uri_without_ssl(self):
-        """Client rejects no ssl when URI is secure."""
+        """Client rejects ssl=None when URI is secure."""
         with self.assertRaises(ValueError) as raised:
             await connect("wss://localhost/", ssl=None)
         self.assertEqual(
@@ -623,6 +931,42 @@ class ClientUsageErrorsTests(unittest.IsolatedAsyncioTestCase):
             "ssl=None is incompatible with a wss:// URI",
         )
 
+    async def test_proxy_ssl_without_https_proxy(self):
+        """Client rejects proxy_ssl when proxy isn't HTTPS."""
+        with self.assertRaises(ValueError) as raised:
+            await connect(
+                "ws://localhost/",
+                proxy="http://localhost:8080",
+                proxy_ssl=True,
+            )
+        self.assertEqual(
+            str(raised.exception),
+            "proxy_ssl argument is incompatible with an http:// proxy",
+        )
+
+    async def test_https_proxy_without_ssl(self):
+        """Client rejects proxy_ssl=None when proxy is HTTPS."""
+        with self.assertRaises(ValueError) as raised:
+            await connect(
+                "ws://localhost/",
+                proxy="https://localhost:8080",
+                proxy_ssl=None,
+            )
+        self.assertEqual(
+            str(raised.exception),
+            "proxy_ssl=None is incompatible with an https:// proxy",
+        )
+
+    async def test_unsupported_proxy(self):
+        """Client rejects unsupported proxy."""
+        with self.assertRaises(InvalidProxy) as raised:
+            async with connect("ws://example.com/", proxy="other://localhost:51080"):
+                self.fail("did not raise")
+        self.assertEqual(
+            str(raised.exception),
+            "other://localhost:51080 isn't a valid proxy: scheme other isn't supported",
+        )
+
     async def test_unix_without_path_or_sock(self):
         """Unix client requires path when sock isn't provided."""
         with self.assertRaises(ValueError) as raised:
