@@ -245,7 +245,7 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         )
 
     async def test_recv_cancellation_before_receiving(self):
-        """recv can be cancelled before receiving a frame."""
+        """recv can be canceled before receiving a frame."""
         recv_task = asyncio.create_task(self.connection.recv())
         await asyncio.sleep(0)  # let the event loop start recv_task
 
@@ -257,7 +257,7 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         self.assertEqual(await self.connection.recv(), "😀")
 
     async def test_recv_cancellation_while_receiving(self):
-        """recv cannot be cancelled after receiving a frame."""
+        """recv cannot be canceled after receiving a frame."""
         recv_task = asyncio.create_task(self.connection.recv())
         await asyncio.sleep(0)  # let the event loop start recv_task
 
@@ -386,7 +386,7 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         )
 
     async def test_recv_streaming_cancellation_before_receiving(self):
-        """recv_streaming can be cancelled before receiving a frame."""
+        """recv_streaming can be canceled before receiving a frame."""
         recv_streaming_task = asyncio.create_task(
             alist(self.connection.recv_streaming())
         )
@@ -403,7 +403,7 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         )
 
     async def test_recv_streaming_cancellation_while_receiving(self):
-        """recv_streaming cannot be cancelled after receiving a frame."""
+        """recv_streaming cannot be canceled after receiving a frame."""
         recv_streaming_task = asyncio.create_task(
             alist(self.connection.recv_streaming())
         )
@@ -866,10 +866,9 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
 
     # Test ping.
 
-    @patch("random.getrandbits")
+    @patch("random.getrandbits", return_value=1918987876)
     async def test_ping(self, getrandbits):
         """ping sends a ping frame with a random payload."""
-        getrandbits.return_value = 1918987876
         await self.connection.ping()
         getrandbits.assert_called_once_with(32)
         await self.assertFrameSent(Frame(Opcode.PING, b"rand"))
@@ -892,6 +891,15 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         async with asyncio_timeout(MS):
             await pong_waiter
 
+    async def test_acknowledge_canceled_ping(self):
+        """ping is acknowledged by a pong with the same payload after being canceled."""
+        async with self.drop_frames_rcvd():  # drop automatic response to ping
+            pong_waiter = await self.connection.ping("this")
+        pong_waiter.cancel()
+        await self.remote_connection.pong("this")
+        with self.assertRaises(asyncio.CancelledError):
+            await pong_waiter
+
     async def test_acknowledge_ping_non_matching_pong(self):
         """ping isn't acknowledged by a pong with a different payload."""
         async with self.drop_frames_rcvd():  # drop automatic response to ping
@@ -902,7 +910,7 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
                 await pong_waiter
 
     async def test_acknowledge_previous_ping(self):
-        """ping is acknowledged by a pong with the same payload as a later ping."""
+        """ping is acknowledged by a pong for a later ping."""
         async with self.drop_frames_rcvd():  # drop automatic response to ping
             pong_waiter = await self.connection.ping("this")
             await self.connection.ping("that")
@@ -910,6 +918,18 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         async with asyncio_timeout(MS):
             await pong_waiter
 
+    async def test_acknowledge_previous_canceled_ping(self):
+        """ping is acknowledged by a pong for a later ping after being canceled."""
+        async with self.drop_frames_rcvd():  # drop automatic response to ping
+            pong_waiter = await self.connection.ping("this")
+            pong_waiter_2 = await self.connection.ping("that")
+        pong_waiter.cancel()
+        await self.remote_connection.pong("that")
+        async with asyncio_timeout(MS):
+            await pong_waiter_2
+        with self.assertRaises(asyncio.CancelledError):
+            await pong_waiter
+
     async def test_ping_duplicate_payload(self):
         """ping rejects the same payload until receiving the pong."""
         async with self.drop_frames_rcvd():  # drop automatic response to ping
@@ -957,17 +977,17 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
 
     # Test keepalive.
 
-    @patch("random.getrandbits")
+    @patch("random.getrandbits", return_value=1918987876)
     async def test_keepalive(self, getrandbits):
         """keepalive sends pings at ping_interval and measures latency."""
-        self.connection.ping_interval = 2 * MS
-        getrandbits.return_value = 1918987876
+        self.connection.ping_interval = 3 * MS
         self.connection.start_keepalive()
+        self.assertIsNotNone(self.connection.keepalive_task)
         self.assertEqual(self.connection.latency, 0)
-        # 2 ms: keepalive() sends a ping frame.
-        # 2.x ms: a pong frame is received.
-        await asyncio.sleep(3 * MS)
-        # 3 ms: check that the ping frame was sent.
+        # 3 ms: keepalive() sends a ping frame.
+        # 3.x ms: a pong frame is received.
+        await asyncio.sleep(4 * MS)
+        # 4 ms: check that the ping frame was sent.
         await self.assertFrameSent(Frame(Opcode.PING, b"rand"))
         self.assertGreater(self.connection.latency, 0)
         self.assertLess(self.connection.latency, MS)
@@ -976,38 +996,35 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         """keepalive is disabled when ping_interval is None."""
         self.connection.ping_interval = None
         self.connection.start_keepalive()
-        await asyncio.sleep(3 * MS)
-        await self.assertNoFrameSent()
+        self.assertIsNone(self.connection.keepalive_task)
 
-    @patch("random.getrandbits")
+    @patch("random.getrandbits", return_value=1918987876)
     async def test_keepalive_times_out(self, getrandbits):
         """keepalive closes the connection if ping_timeout elapses."""
         self.connection.ping_interval = 4 * MS
         self.connection.ping_timeout = 2 * MS
         async with self.drop_frames_rcvd():
-            getrandbits.return_value = 1918987876
             self.connection.start_keepalive()
             # 4 ms: keepalive() sends a ping frame.
             await asyncio.sleep(4 * MS)
-            # Exiting the context manager sleeps for MS.
+            # Exiting the context manager sleeps for 1 ms.
             # 4.x ms: a pong frame is dropped.
         # 6 ms: no pong frame is received; the connection is closed.
         await asyncio.sleep(2 * MS)
         # 7 ms: check that the connection is closed.
         self.assertEqual(self.connection.state, State.CLOSED)
 
-    @patch("random.getrandbits")
+    @patch("random.getrandbits", return_value=1918987876)
     async def test_keepalive_ignores_timeout(self, getrandbits):
         """keepalive ignores timeouts if ping_timeout isn't set."""
         self.connection.ping_interval = 4 * MS
         self.connection.ping_timeout = None
         async with self.drop_frames_rcvd():
-            getrandbits.return_value = 1918987876
             self.connection.start_keepalive()
             # 4 ms: keepalive() sends a ping frame.
-            await asyncio.sleep(4 * MS)
-            # Exiting the context manager sleeps for MS.
             # 4.x ms: a pong frame is dropped.
+            await asyncio.sleep(4 * MS)
+            # Exiting the context manager sleeps for 1 ms.
         # 6 ms: no pong frame is received; the connection remains open.
         await asyncio.sleep(2 * MS)
         # 7 ms: check that the connection is still open.
@@ -1015,7 +1032,7 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
 
     async def test_keepalive_terminates_while_sleeping(self):
         """keepalive task terminates while waiting to send a ping."""
-        self.connection.ping_interval = 2 * MS
+        self.connection.ping_interval = 3 * MS
         self.connection.start_keepalive()
         await asyncio.sleep(MS)
         await self.connection.close()
@@ -1023,15 +1040,15 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
 
     async def test_keepalive_terminates_while_waiting_for_pong(self):
         """keepalive task terminates while waiting to receive a pong."""
-        self.connection.ping_interval = 2 * MS
-        self.connection.ping_timeout = 2 * MS
+        self.connection.ping_interval = MS
+        self.connection.ping_timeout = 3 * MS
         async with self.drop_frames_rcvd():
             self.connection.start_keepalive()
-            # 2 ms: keepalive() sends a ping frame.
-            await asyncio.sleep(2 * MS)
-            # Exiting the context manager sleeps for MS.
-            # 2.x ms: a pong frame is dropped.
-        # 3 ms: close the connection before ping_timeout elapses.
+            # 1 ms: keepalive() sends a ping frame.
+            # 1.x ms: a pong frame is dropped.
+            await asyncio.sleep(MS)
+            # Exiting the context manager sleeps for 1 ms.
+        # 2 ms: close the connection before ping_timeout elapses.
         await self.connection.close()
         self.assertTrue(self.connection.keepalive_task.done())
 
@@ -1041,9 +1058,9 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         async with self.drop_frames_rcvd():
             self.connection.start_keepalive()
             # 2 ms: keepalive() sends a ping frame.
-            await asyncio.sleep(2 * MS)
-            # Exiting the context manager sleeps for MS.
             # 2.x ms: a pong frame is dropped.
+            await asyncio.sleep(2 * MS)
+            # Exiting the context manager sleeps for 1 ms.
         # 3 ms: inject a fault: raise an exception in the pending pong waiter.
         pong_waiter = next(iter(self.connection.pong_waiters.values()))[0]
         with self.assertLogs("websockets", logging.ERROR) as logs:
@@ -1121,17 +1138,13 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         """Connection has a logger attribute."""
         self.assertIsInstance(self.connection.logger, logging.LoggerAdapter)
 
-    @unittest.mock.patch(
-        "asyncio.BaseTransport.get_extra_info", return_value=("sock", 1234)
-    )
+    @patch("asyncio.BaseTransport.get_extra_info", return_value=("sock", 1234))
     async def test_local_address(self, get_extra_info):
         """Connection provides a local_address attribute."""
         self.assertEqual(self.connection.local_address, ("sock", 1234))
         get_extra_info.assert_called_with("sockname")
 
-    @unittest.mock.patch(
-        "asyncio.BaseTransport.get_extra_info", return_value=("peer", 1234)
-    )
+    @patch("asyncio.BaseTransport.get_extra_info", return_value=("peer", 1234))
     async def test_remote_address(self, get_extra_info):
         """Connection provides a remote_address attribute."""
         self.assertEqual(self.connection.remote_address, ("peer", 1234))
@@ -1192,12 +1205,11 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
 
     # Test safety nets — catching all exceptions in case of bugs.
 
-    @patch("websockets.protocol.Protocol.events_received")
+    # Inject a fault in a random call in data_received().
+    # This test is tightly coupled to the implementation.
+    @patch("websockets.protocol.Protocol.events_received", side_effect=AssertionError)
     async def test_unexpected_failure_in_data_received(self, events_received):
         """Unexpected internal error in data_received() is correctly reported."""
-        # Inject a fault in a random call in data_received().
-        # This test is tightly coupled to the implementation.
-        events_received.side_effect = AssertionError
         # Receive a message to trigger the fault.
         await self.remote_connection.send("😀")
 
@@ -1208,13 +1220,11 @@ class ClientConnectionTests(AssertNoLogsMixin, unittest.IsolatedAsyncioTestCase)
         self.assertEqual(str(exc), "no close frame received or sent")
         self.assertIsInstance(exc.__cause__, AssertionError)
 
-    @patch("websockets.protocol.Protocol.send_text")
+    # Inject a fault in a random call in send_context().
+    # This test is tightly coupled to the implementation.
+    @patch("websockets.protocol.Protocol.send_text", side_effect=AssertionError)
     async def test_unexpected_failure_in_send_context(self, send_text):
         """Unexpected internal error in send_context() is correctly reported."""
-        # Inject a fault in a random call in send_context().
-        # This test is tightly coupled to the implementation.
-        send_text.side_effect = AssertionError
-
         # Send a message to trigger the fault.
         # The connection closed exception reports the injected fault.
         with self.assertRaises(ConnectionClosedError) as raised:
