@@ -59,16 +59,15 @@ class ServerTests(EvalShellMixin, AssertNoLogsMixin, unittest.IsolatedAsyncioTes
                     await client.recv()
                 self.assertEqual(
                     str(raised.exception),
-                    "received 1011 (internal error); "
-                    "then sent 1011 (internal error)",
+                    "received 1011 (internal error); then sent 1011 (internal error)",
                 )
 
     async def test_existing_socket(self):
         """Server receives connection using a pre-existing socket."""
         with socket.create_server(("localhost", 0)) as sock:
-            async with serve(handler, sock=sock, host=None, port=None):
-                uri = "ws://{}:{}/".format(*sock.getsockname())
-                async with connect(uri) as client:
+            host, port = sock.getsockname()
+            async with serve(handler, sock=sock):
+                async with connect(f"ws://{host}:{port}/") as client:
                     await self.assertEval(client, "ws.protocol.state.name", "OPEN")
 
     async def test_select_subprotocol(self):
@@ -118,6 +117,22 @@ class ServerTests(EvalShellMixin, AssertNoLogsMixin, unittest.IsolatedAsyncioTes
                 "server rejected WebSocket connection: HTTP 500",
             )
 
+    async def test_compression_is_enabled(self):
+        """Server enables compression by default."""
+        async with serve(*args) as server:
+            async with connect(get_uri(server)) as client:
+                await self.assertEval(
+                    client,
+                    "[type(ext).__name__ for ext in ws.protocol.extensions]",
+                    "['PerMessageDeflate']",
+                )
+
+    async def test_disable_compression(self):
+        """Server disables compression."""
+        async with serve(*args, compression=None) as server:
+            async with connect(get_uri(server)) as client:
+                await self.assertEval(client, "ws.protocol.extensions", "[]")
+
     async def test_process_request_returns_none(self):
         """Server runs process_request and continues the handshake."""
 
@@ -360,22 +375,6 @@ class ServerTests(EvalShellMixin, AssertNoLogsMixin, unittest.IsolatedAsyncioTes
                     client, "'Server' in ws.response.headers", "False"
                 )
 
-    async def test_compression_is_enabled(self):
-        """Server enables compression by default."""
-        async with serve(*args) as server:
-            async with connect(get_uri(server)) as client:
-                await self.assertEval(
-                    client,
-                    "[type(ext).__name__ for ext in ws.protocol.extensions]",
-                    "['PerMessageDeflate']",
-                )
-
-    async def test_disable_compression(self):
-        """Server disables compression."""
-        async with serve(*args, compression=None) as server:
-            async with connect(get_uri(server)) as client:
-                await self.assertEval(client, "ws.protocol.extensions", "[]")
-
     async def test_keepalive_is_enabled(self):
         """Server enables keepalive and measures latency."""
         async with serve(*args, ping_interval=MS) as server:
@@ -473,6 +472,10 @@ class ServerTests(EvalShellMixin, AssertNoLogsMixin, unittest.IsolatedAsyncioTes
         )
         self.assertEqual(
             [str(record.exc_info[1]) for record in logs.records],
+            ["did not receive a valid HTTP request"],
+        )
+        self.assertEqual(
+            [str(record.exc_info[1].__cause__) for record in logs.records],
             ["invalid HTTP request line: HELO relay.invalid"],
         )
 
@@ -539,7 +542,8 @@ class ServerTests(EvalShellMixin, AssertNoLogsMixin, unittest.IsolatedAsyncioTes
                     async with asyncio_timeout(2 * MS):
                         await server.wait_closed()
 
-                async with asyncio_timeout(3 * MS):
+                # Set a large timeout here, else the test becomes flaky.
+                async with asyncio_timeout(5 * MS):
                     await server.wait_closed()
 
 
