@@ -645,10 +645,9 @@ class ClientConnectionTests(unittest.TestCase):
 
     # Test ping.
 
-    @patch("random.getrandbits")
+    @patch("random.getrandbits", return_value=1918987876)
     def test_ping(self, getrandbits):
         """ping sends a ping frame with a random payload."""
-        getrandbits.return_value = 1918987876
         self.connection.ping()
         getrandbits.assert_called_once_with(32)
         self.assertFrameSent(Frame(Opcode.PING, b"rand"))
@@ -678,13 +677,22 @@ class ClientConnectionTests(unittest.TestCase):
         self.assertFalse(pong_waiter.wait(MS))
 
     def test_acknowledge_previous_ping(self):
-        """ping is acknowledged by a pong with the same payload as a later ping."""
+        """ping is acknowledged by a pong for as a later ping."""
         with self.drop_frames_rcvd():  # drop automatic response to ping
             pong_waiter = self.connection.ping("this")
             self.connection.ping("that")
         self.remote_connection.pong("that")
         self.assertTrue(pong_waiter.wait(MS))
 
+    def test_acknowledge_ping_on_close(self):
+        """ping with ack_on_close is acknowledged when the connection is closed."""
+        with self.drop_frames_rcvd():  # drop automatic response to ping
+            pong_waiter_ack_on_close = self.connection.ping("this", ack_on_close=True)
+            pong_waiter = self.connection.ping("that")
+        self.connection.close()
+        self.assertTrue(pong_waiter_ack_on_close.wait(MS))
+        self.assertFalse(pong_waiter.wait(MS))
+
     def test_ping_duplicate_payload(self):
         """ping rejects the same payload until receiving the pong."""
         with self.drop_frames_rcvd():  # drop automatic response to ping
@@ -729,6 +737,113 @@ class ClientConnectionTests(unittest.TestCase):
         with self.assertRaises(TypeError):
             self.connection.pong([])
 
+    # Test keepalive.
+
+    @patch("random.getrandbits", return_value=1918987876)
+    def test_keepalive(self, getrandbits):
+        """keepalive sends pings at ping_interval and measures latency."""
+        self.connection.ping_interval = 4 * MS
+        self.connection.start_keepalive()
+        self.assertIsNotNone(self.connection.keepalive_thread)
+        self.assertEqual(self.connection.latency, 0)
+        # 3 ms: keepalive() sends a ping frame.
+        # 3.x ms: a pong frame is received.
+        time.sleep(4 * MS)
+        # 4 ms: check that the ping frame was sent.
+        self.assertFrameSent(Frame(Opcode.PING, b"rand"))
+        self.assertGreater(self.connection.latency, 0)
+        self.assertLess(self.connection.latency, MS)
+
+    def test_disable_keepalive(self):
+        """keepalive is disabled when ping_interval is None."""
+        self.connection.ping_interval = None
+        self.connection.start_keepalive()
+        self.assertIsNone(self.connection.keepalive_thread)
+
+    @patch("random.getrandbits", return_value=1918987876)
+    def test_keepalive_times_out(self, getrandbits):
+        """keepalive closes the connection if ping_timeout elapses."""
+        self.connection.ping_interval = 4 * MS
+        self.connection.ping_timeout = 2 * MS
+        with self.drop_frames_rcvd():
+            self.connection.start_keepalive()
+            # 4 ms: keepalive() sends a ping frame.
+            time.sleep(4 * MS)
+            # Exiting the context manager sleeps for 1 ms.
+            # 4.x ms: a pong frame is dropped.
+        # 6 ms: no pong frame is received; the connection is closed.
+        time.sleep(2 * MS)
+        # 7 ms: check that the connection is closed.
+        self.assertEqual(self.connection.state, State.CLOSED)
+
+    @patch("random.getrandbits", return_value=1918987876)
+    def test_keepalive_ignores_timeout(self, getrandbits):
+        """keepalive ignores timeouts if ping_timeout isn't set."""
+        self.connection.ping_interval = 4 * MS
+        self.connection.ping_timeout = None
+        with self.drop_frames_rcvd():
+            self.connection.start_keepalive()
+            # 4 ms: keepalive() sends a ping frame.
+            time.sleep(4 * MS)
+            # Exiting the context manager sleeps for 1 ms.
+            # 4.x ms: a pong frame is dropped.
+        # 6 ms: no pong frame is received; the connection remains open.
+        time.sleep(2 * MS)
+        # 7 ms: check that the connection is still open.
+        self.assertEqual(self.connection.state, State.OPEN)
+
+    def test_keepalive_terminates_while_sleeping(self):
+        """keepalive task terminates while waiting to send a ping."""
+        self.connection.ping_interval = 3 * MS
+        self.connection.start_keepalive()
+        time.sleep(MS)
+        self.connection.close()
+        self.connection.keepalive_thread.join(MS)
+        self.assertFalse(self.connection.keepalive_thread.is_alive())
+
+    def test_keepalive_terminates_when_sending_ping_fails(self):
+        """keepalive task terminates when sending a ping fails."""
+        self.connection.ping_interval = 1 * MS
+        self.connection.start_keepalive()
+        with self.drop_eof_rcvd(), self.drop_frames_rcvd():
+            self.connection.close()
+        self.assertFalse(self.connection.keepalive_thread.is_alive())
+
+    def test_keepalive_terminates_while_waiting_for_pong(self):
+        """keepalive task terminates while waiting to receive a pong."""
+        self.connection.ping_interval = MS
+        self.connection.ping_timeout = 4 * MS
+        with self.drop_frames_rcvd():
+            self.connection.start_keepalive()
+            # 1 ms: keepalive() sends a ping frame.
+            # 1.x ms: a pong frame is dropped.
+            time.sleep(MS)
+            # Exiting the context manager sleeps for 1 ms.
+        # 2 ms: close the connection before ping_timeout elapses.
+        self.connection.close()
+        self.connection.keepalive_thread.join(MS)
+        self.assertFalse(self.connection.keepalive_thread.is_alive())
+
+    def test_keepalive_reports_errors(self):
+        """keepalive reports unexpected errors in logs."""
+        self.connection.ping_interval = 2 * MS
+        with self.drop_frames_rcvd():
+            self.connection.start_keepalive()
+            # 2 ms: keepalive() sends a ping frame.
+            # 2.x ms: a pong frame is dropped.
+            with self.assertLogs("websockets", logging.ERROR) as logs:
+                with patch("threading.Event.wait", side_effect=Exception("BOOM")):
+                    time.sleep(3 * MS)
+            # Exiting the context manager sleeps for 1 ms.
+        self.assertEqual(
+            [record.getMessage() for record in logs.records],
+            ["keepalive ping failed"],
+        )
+        self.assertEqual(
+            [str(record.exc_info[1]) for record in logs.records],
+            ["BOOM"],
+        )
+
     # Test parameters.
 
     def test_close_timeout(self):
@@ -791,13 +906,13 @@ class ClientConnectionTests(unittest.TestCase):
         """Connection has a logger attribute."""
         self.assertIsInstance(self.connection.logger, logging.LoggerAdapter)
 
-    @unittest.mock.patch("socket.socket.getsockname", return_value=("sock", 1234))
+    @patch("socket.socket.getsockname", return_value=("sock", 1234))
     def test_local_address(self, getsockname):
         """Connection provides a local_address attribute."""
         self.assertEqual(self.connection.local_address, ("sock", 1234))
         getsockname.assert_called_with()
 
-    @unittest.mock.patch("socket.socket.getpeername", return_value=("peer", 1234))
+    @patch("socket.socket.getpeername", return_value=("peer", 1234))
     def test_remote_address(self, getpeername):
         """Connection provides a remote_address attribute."""
         self.assertEqual(self.connection.remote_address, ("peer", 1234))
@@ -865,12 +980,11 @@ class ClientConnectionTests(unittest.TestCase):
 
     # Test safety nets — catching all exceptions in case of bugs.
 
-    @patch("websockets.protocol.Protocol.events_received")
+    # Inject a fault in a random call in recv_events().
+    # This test is tightly coupled to the implementation.
+    @patch("websockets.protocol.Protocol.events_received", side_effect=AssertionError)
     def test_unexpected_failure_in_recv_events(self, events_received):
         """Unexpected internal error in recv_events() is correctly reported."""
-        # Inject a fault in a random call in recv_events().
-        # This test is tightly coupled to the implementation.
-        events_received.side_effect = AssertionError
         # Receive a message to trigger the fault.
         self.remote_connection.send("😀")
 
@@ -881,13 +995,11 @@ class ClientConnectionTests(unittest.TestCase):
         self.assertEqual(str(exc), "no close frame received or sent")
         self.assertIsInstance(exc.__cause__, AssertionError)
 
-    @patch("websockets.protocol.Protocol.send_text")
+    # Inject a fault in a random call in send_context().
+    # This test is tightly coupled to the implementation.
+    @patch("websockets.protocol.Protocol.send_text", side_effect=AssertionError)
     def test_unexpected_failure_in_send_context(self, send_text):
         """Unexpected internal error in send_context() is correctly reported."""
-        # Inject a fault in a random call in send_context().
-        # This test is tightly coupled to the implementation.
-        send_text.side_effect = AssertionError
-
         # Send a message to trigger the fault.
         # The connection closed exception reports the injected fault.
         with self.assertRaises(ConnectionClosedError) as raised:
