@@ -58,16 +58,15 @@ class ServerTests(EvalShellMixin, unittest.TestCase):
                     client.recv()
                 self.assertEqual(
                     str(raised.exception),
-                    "received 1011 (internal error); "
-                    "then sent 1011 (internal error)",
+                    "received 1011 (internal error); then sent 1011 (internal error)",
                 )
 
     def test_existing_socket(self):
         """Server receives connection using a pre-existing socket."""
         with socket.create_server(("localhost", 0)) as sock:
+            host, port = sock.getsockname()
             with run_server(sock=sock):
-                uri = "ws://{}:{}/".format(*sock.getsockname())
-                with connect(uri) as client:
+                with connect(f"ws://{host}:{port}/") as client:
                     self.assertEval(client, "ws.protocol.state.name", "OPEN")
 
     def test_select_subprotocol(self):
@@ -116,6 +115,22 @@ class ServerTests(EvalShellMixin, unittest.TestCase):
                 "server rejected WebSocket connection: HTTP 500",
             )
 
+    def test_compression_is_enabled(self):
+        """Server enables compression by default."""
+        with run_server() as server:
+            with connect(get_uri(server)) as client:
+                self.assertEval(
+                    client,
+                    "[type(ext).__name__ for ext in ws.protocol.extensions]",
+                    "['PerMessageDeflate']",
+                )
+
+    def test_disable_compression(self):
+        """Server disables compression."""
+        with run_server(compression=None) as server:
+            with connect(get_uri(server)) as client:
+                self.assertEval(client, "ws.protocol.extensions", "[]")
+
     def test_process_request_returns_none(self):
         """Server runs process_request and continues the handshake."""
 
@@ -221,21 +236,26 @@ class ServerTests(EvalShellMixin, unittest.TestCase):
             with connect(get_uri(server)) as client:
                 self.assertEval(client, "'Server' in ws.response.headers", "False")
 
-    def test_compression_is_enabled(self):
-        """Server enables compression by default."""
-        with run_server() as server:
+    def test_keepalive_is_enabled(self):
+        """Server enables keepalive and measures latency."""
+        with run_server(ping_interval=MS) as server:
             with connect(get_uri(server)) as client:
-                self.assertEval(
-                    client,
-                    "[type(ext).__name__ for ext in ws.protocol.extensions]",
-                    "['PerMessageDeflate']",
-                )
+                client.send("ws.latency")
+                latency = eval(client.recv())
+                self.assertEqual(latency, 0)
+                time.sleep(2 * MS)
+                client.send("ws.latency")
+                latency = eval(client.recv())
+                self.assertGreater(latency, 0)
 
-    def test_disable_compression(self):
-        """Server disables compression."""
-        with run_server(compression=None) as server:
+    def test_disable_keepalive(self):
+        """Server disables keepalive."""
+        with run_server(ping_interval=None) as server:
             with connect(get_uri(server)) as client:
-                self.assertEval(client, "ws.protocol.extensions", "[]")
+                time.sleep(2 * MS)
+                client.send("ws.latency")
+                latency = eval(client.recv())
+                self.assertEqual(latency, 0)
 
     def test_logger(self):
         """Server accepts a logger argument."""
@@ -311,6 +331,10 @@ class ServerTests(EvalShellMixin, unittest.TestCase):
         )
         self.assertEqual(
             [str(record.exc_info[1]) for record in logs.records],
+            ["did not receive a valid HTTP request"],
+        )
+        self.assertEqual(
+            [str(record.exc_info[1].__cause__) for record in logs.records],
             ["invalid HTTP request line: HELO relay.invalid"],
         )
 
