@@ -8,7 +8,12 @@ from unittest.mock import patch
 from websockets.client import *
 from websockets.client import backoff
 from websockets.datastructures import Headers
-from websockets.exceptions import InvalidHandshake, InvalidHeader, InvalidStatus
+from websockets.exceptions import (
+    InvalidHandshake,
+    InvalidHeader,
+    InvalidMessage,
+    InvalidStatus,
+)
 from websockets.frames import OP_TEXT, Frame
 from websockets.http11 import Request, Response
 from websockets.protocol import CONNECTING, OPEN
@@ -199,7 +204,7 @@ class ResponseTests(unittest.TestCase):
                 }
             ),
         )
-        self.assertIsNone(response.body)
+        self.assertEqual(response.body, b"")
         self.assertIsNone(client.handshake_exc)
 
     def test_receive_failed_response(self, _generate_key):
@@ -244,9 +249,14 @@ class ResponseTests(unittest.TestCase):
         client.receive_eof()
 
         self.assertEqual(client.events_received(), [])
-        self.assertIsInstance(client.handshake_exc, EOFError)
+        self.assertIsInstance(client.handshake_exc, InvalidMessage)
         self.assertEqual(
             str(client.handshake_exc),
+            "did not receive a valid HTTP response",
+        )
+        self.assertIsInstance(client.handshake_exc.__cause__, EOFError)
+        self.assertEqual(
+            str(client.handshake_exc.__cause__),
             "connection closed while reading HTTP status line",
         )
 
@@ -257,9 +267,14 @@ class ResponseTests(unittest.TestCase):
         client.receive_eof()
 
         self.assertEqual(client.events_received(), [])
-        self.assertIsInstance(client.handshake_exc, EOFError)
+        self.assertIsInstance(client.handshake_exc, InvalidMessage)
         self.assertEqual(
             str(client.handshake_exc),
+            "did not receive a valid HTTP response",
+        )
+        self.assertIsInstance(client.handshake_exc.__cause__, EOFError)
+        self.assertEqual(
+            str(client.handshake_exc.__cause__),
             "connection closed while reading HTTP headers",
         )
 
@@ -272,9 +287,14 @@ class ResponseTests(unittest.TestCase):
         client.receive_data(b"250 Ok\r\n")
 
         self.assertEqual(client.events_received(), [])
-        self.assertIsInstance(client.handshake_exc, ValueError)
+        self.assertIsInstance(client.handshake_exc, InvalidMessage)
         self.assertEqual(
             str(client.handshake_exc),
+            "did not receive a valid HTTP response",
+        )
+        self.assertIsInstance(client.handshake_exc.__cause__, ValueError)
+        self.assertEqual(
+            str(client.handshake_exc.__cause__),
             "invalid HTTP status line: 220 smtp.invalid",
         )
 
@@ -618,8 +638,7 @@ class HandshakeTests(unittest.TestCase):
         self.assertHandshakeError(
             client,
             InvalidHandshake,
-            "invalid Sec-WebSocket-Protocol header: "
-            "multiple values: superchat, chat",
+            "invalid Sec-WebSocket-Protocol header: multiple values: superchat, chat",
         )
 
 
