@@ -1,5 +1,6 @@
 import http
 import logging
+import re
 import sys
 import unittest
 from unittest.mock import patch
@@ -7,6 +8,7 @@ from unittest.mock import patch
 from websockets.datastructures import Headers
 from websockets.exceptions import (
     InvalidHeader,
+    InvalidMessage,
     InvalidOrigin,
     InvalidUpgrade,
     NegotiationError,
@@ -207,9 +209,15 @@ class RequestTests(unittest.TestCase):
         server.receive_eof()
 
         self.assertEqual(server.events_received(), [])
-        self.assertIsInstance(server.handshake_exc, EOFError)
+        self.assertEqual(server.events_received(), [])
+        self.assertIsInstance(server.handshake_exc, InvalidMessage)
         self.assertEqual(
             str(server.handshake_exc),
+            "did not receive a valid HTTP request",
+        )
+        self.assertIsInstance(server.handshake_exc.__cause__, EOFError)
+        self.assertEqual(
+            str(server.handshake_exc.__cause__),
             "connection closed while reading HTTP request line",
         )
 
@@ -220,9 +228,14 @@ class RequestTests(unittest.TestCase):
         server.receive_eof()
 
         self.assertEqual(server.events_received(), [])
-        self.assertIsInstance(server.handshake_exc, EOFError)
+        self.assertIsInstance(server.handshake_exc, InvalidMessage)
         self.assertEqual(
             str(server.handshake_exc),
+            "did not receive a valid HTTP request",
+        )
+        self.assertIsInstance(server.handshake_exc.__cause__, EOFError)
+        self.assertEqual(
+            str(server.handshake_exc.__cause__),
             "connection closed while reading HTTP headers",
         )
 
@@ -233,10 +246,14 @@ class RequestTests(unittest.TestCase):
         server.receive_data(b"MAIL FROM: <alice@invalid>\r\n")
         server.receive_data(b"RCPT TO: <bob@invalid>\r\n")
 
-        self.assertEqual(server.events_received(), [])
-        self.assertIsInstance(server.handshake_exc, ValueError)
+        self.assertIsInstance(server.handshake_exc, InvalidMessage)
         self.assertEqual(
             str(server.handshake_exc),
+            "did not receive a valid HTTP request",
+        )
+        self.assertIsInstance(server.handshake_exc.__cause__, ValueError)
+        self.assertEqual(
+            str(server.handshake_exc.__cause__),
             "invalid HTTP request line: HELO relay.invalid",
         )
 
@@ -265,7 +282,7 @@ class ResponseTests(unittest.TestCase):
                 }
             ),
         )
-        self.assertIsNone(response.body)
+        self.assertEqual(response.body, b"")
 
     @patch("email.utils.formatdate", return_value=DATE)
     def test_reject_response(self, _formatdate):
@@ -297,12 +314,14 @@ class ResponseTests(unittest.TestCase):
         self.assertEqual(response.status_code, 404)
         self.assertEqual(response.reason_phrase, "Not Found")
 
-    @patch("websockets.server.ServerProtocol.process_request")
+    @patch(
+        "websockets.server.ServerProtocol.process_request",
+        side_effect=Exception("BOOM"),
+    )
     def test_unexpected_error(self, process_request):
         """accept() handles unexpected errors and returns an error response."""
         server = ServerProtocol()
         request = make_request()
-        process_request.side_effect = (Exception("BOOM"),)
         response = server.accept(request)
 
         self.assertEqual(response.status_code, 500)
@@ -591,7 +610,7 @@ class HandshakeTests(unittest.TestCase):
         self.assertEqual(server.origin, "https://other.example.com")
 
     def test_unsupported_origin(self):
-        """Handshake succeeds when checking origins and the origin is unsupported."""
+        """Handshake fails when checking origins and the origin is unsupported."""
         server = ServerProtocol(
             origins=["https://example.com", "https://other.example.com"]
         )
@@ -607,6 +626,53 @@ class HandshakeTests(unittest.TestCase):
             "invalid Origin header: https://original.example.com",
         )
 
+    def test_supported_origin_regex(self):
+        """Handshake succeeds when checking origins and the origin is supported."""
+        server = ServerProtocol(
+            origins=[re.compile(r"https://(?!original)[a-z]+\.example\.com")]
+        )
+        request = make_request()
+        request.headers["Origin"] = "https://other.example.com"
+        response = server.accept(request)
+        server.send_response(response)
+
+        self.assertHandshakeSuccess(server)
+        self.assertEqual(server.origin, "https://other.example.com")
+
+    def test_unsupported_origin_regex(self):
+        """Handshake fails when checking origins and the origin is unsupported."""
+        server = ServerProtocol(
+            origins=[re.compile(r"https://(?!original)[a-z]+\.example\.com")]
+        )
+        request = make_request()
+        request.headers["Origin"] = "https://original.example.com"
+        response = server.accept(request)
+        server.send_response(response)
+
+        self.assertEqual(response.status_code, 403)
+        self.assertHandshakeError(
+            server,
+            InvalidOrigin,
+            "invalid Origin header: https://original.example.com",
+        )
+
+    def test_partial_match_origin_regex(self):
+        """Handshake fails when checking origins and the origin a partial match."""
+        server = ServerProtocol(
+            origins=[re.compile(r"https://(?!original)[a-z]+\.example\.com")]
+        )
+        request = make_request()
+        request.headers["Origin"] = "https://other.example.com.hacked"
+        response = server.accept(request)
+        server.send_response(response)
+
+        self.assertEqual(response.status_code, 403)
+        self.assertHandshakeError(
+            server,
+            InvalidOrigin,
+            "invalid Origin header: https://other.example.com.hacked",
+        )
+
     def test_no_origin_accepted(self):
         """Handshake succeeds when the lack of an origin is accepted."""
         server = ServerProtocol(origins=[None])
